@@ -6,8 +6,9 @@ use serde::{Deserialize, Serialize};
 use solana_program::{
     instruction::{AccountMeta, Instruction},
     pubkey::Pubkey,
+    sysvar::rent,
 };
-use std::{cmp::max, convert::TryInto};
+use std::convert::TryInto;
 
 use arrayref::{array_ref, array_refs};
 use num_enum::{IntoPrimitive, TryFromPrimitive};
@@ -330,6 +331,8 @@ pub enum MarketInstruction {
     /// 7. `[]` coin currency Mint
     /// 8. `[]` price currency Mint
     /// 9. `[]` the rent sysvar
+    /// 10. `[]` open orders market authority (optional)
+    /// 11. `[]` prune authority (optional, requires open orders market authority)
     InitializeMarket(InitializeMarketInstruction),
     /// 0. `[writable]` the market
     /// 1. `[writable]` the OpenOrders account to use
@@ -432,6 +435,27 @@ pub enum MarketInstruction {
     /// 3. `[writable]` OpenOrders
     /// 4. `[]`
     SendTake(SendTakeInstruction),
+    /// 0. `[writable]` OpenOrders
+    /// 1. `[signer]` the OpenOrders owner
+    /// 2. `[writable]` the destination account to send rent exemption SOL to
+    /// 3. `[]` market
+    CloseOpenOrders,
+    /// 0. `[writable]` OpenOrders
+    /// 1. `[signer]` the OpenOrders owner
+    /// 2. `[]` market
+    /// 3. `[]` the rent sysvar
+    /// 4. `[signer]` open orders market authority (optional).
+    InitOpenOrders,
+    /// Removes all orders for a given open orders account from the orderbook.
+    ///
+    /// 0. `[writable]` market
+    /// 1. `[writable]` bids
+    /// 2. `[writable]` asks
+    /// 3. `[signer]` prune authority
+    /// 4. `[]` open orders.
+    /// 5. `[]` open orders owner.
+    /// 6. `[writable]` event queue.
+    Prune(u16),
 }
 
 impl MarketInstruction {
@@ -523,6 +547,12 @@ impl MarketInstruction {
                 let data_arr = array_ref![data, 0, 46];
                 SendTakeInstruction::unpack(data_arr)?
             }),
+            (14, 0) => MarketInstruction::CloseOpenOrders,
+            (15, 0) => MarketInstruction::InitOpenOrders,
+            (16, 2) => {
+                let limit = array_ref![data, 0, 2];
+                MarketInstruction::Prune(u16::from_le_bytes(*limit))
+            }
             _ => return None,
         })
     }
@@ -545,6 +575,8 @@ pub fn initialize_market(
     pc_mint_pk: &Pubkey,
     coin_vault_pk: &Pubkey,
     pc_vault_pk: &Pubkey,
+    authority_pk: Option<&Pubkey>,
+    prune_authority_pk: Option<&Pubkey>,
     // srm_vault_pk: &Pubkey,
     bids_pk: &Pubkey,
     asks_pk: &Pubkey,
@@ -579,7 +611,7 @@ pub fn initialize_market(
 
     let rent_sysvar = AccountMeta::new_readonly(solana_program::sysvar::rent::ID, false);
 
-    let accounts = vec![
+    let mut accounts = vec![
         market_account,
         req_q,
         event_q,
@@ -593,6 +625,14 @@ pub fn initialize_market(
         //srm_mint,
         rent_sysvar,
     ];
+    if let Some(auth) = authority_pk {
+        let authority = AccountMeta::new_readonly(*auth, false);
+        accounts.push(authority);
+        if let Some(prune_auth) = prune_authority_pk {
+            let authority = AccountMeta::new_readonly(*prune_auth, false);
+            accounts.push(authority);
+        }
+    }
 
     Ok(Instruction {
         program_id: *program_id,
@@ -623,7 +663,7 @@ pub fn new_order(
     client_order_id: u64,
     self_trade_behavior: SelfTradeBehavior,
     limit: u16,
-    max_native_pc_qty_including_fees: NonZeroU64
+    max_native_pc_qty_including_fees: NonZeroU64,
 ) -> Result<Instruction, DexError> {
     let data = MarketInstruction::NewOrderV3(NewOrderInstructionV3 {
         side,
@@ -633,7 +673,7 @@ pub fn new_order(
         client_order_id,
         self_trade_behavior,
         limit,
-        max_native_pc_qty_including_fees
+        max_native_pc_qty_including_fees,
     })
     .pack();
     let mut accounts = vec![
@@ -726,15 +766,11 @@ pub fn cancel_order(
     side: Side,
     order_id: u128,
 ) -> Result<Instruction, DexError> {
-    let data = MarketInstruction::CancelOrderV2(CancelOrderInstructionV2 {
-        side,
-        order_id,
-    })
-    .pack();
+    let data = MarketInstruction::CancelOrderV2(CancelOrderInstructionV2 { side, order_id }).pack();
     let accounts: Vec<AccountMeta> = vec![
-        AccountMeta::new_readonly(*market, false),
-        AccountMeta::new_readonly(*market_bids, false),
-        AccountMeta::new_readonly(*market_asks, false),
+        AccountMeta::new(*market, false),
+        AccountMeta::new(*market_bids, false),
+        AccountMeta::new(*market_asks, false),
         AccountMeta::new(*open_orders_account, false),
         AccountMeta::new_readonly(*open_orders_account_owner, true),
         AccountMeta::new(*event_queue, false),
@@ -793,9 +829,9 @@ pub fn cancel_order_by_client_order_id(
 ) -> Result<Instruction, DexError> {
     let data = MarketInstruction::CancelOrderByClientIdV2(client_order_id).pack();
     let accounts: Vec<AccountMeta> = vec![
-        AccountMeta::new_readonly(*market, false),
-        AccountMeta::new_readonly(*market_bids, false),
-        AccountMeta::new_readonly(*market_asks, false),
+        AccountMeta::new(*market, false),
+        AccountMeta::new(*market_bids, false),
+        AccountMeta::new(*market_asks, false),
         AccountMeta::new(*open_orders_account, false),
         AccountMeta::new_readonly(*open_orders_account_owner, true),
         AccountMeta::new(*event_queue, false),
@@ -849,6 +885,79 @@ pub fn sweep_fees(
     })
 }
 
+pub fn close_open_orders(
+    program_id: &Pubkey,
+    open_orders: &Pubkey,
+    owner: &Pubkey,
+    destination: &Pubkey,
+    market: &Pubkey,
+) -> Result<Instruction, DexError> {
+    let data = MarketInstruction::CloseOpenOrders.pack();
+    let accounts: Vec<AccountMeta> = vec![
+        AccountMeta::new(*open_orders, false),
+        AccountMeta::new_readonly(*owner, true),
+        AccountMeta::new(*destination, false),
+        AccountMeta::new_readonly(*market, false),
+    ];
+    Ok(Instruction {
+        program_id: *program_id,
+        data,
+        accounts,
+    })
+}
+
+pub fn init_open_orders(
+    program_id: &Pubkey,
+    open_orders: &Pubkey,
+    owner: &Pubkey,
+    market: &Pubkey,
+    market_authority: Option<&Pubkey>,
+) -> Result<Instruction, DexError> {
+    let data = MarketInstruction::InitOpenOrders.pack();
+    let mut accounts: Vec<AccountMeta> = vec![
+        AccountMeta::new(*open_orders, false),
+        AccountMeta::new_readonly(*owner, true),
+        AccountMeta::new_readonly(*market, false),
+        AccountMeta::new_readonly(rent::ID, false),
+    ];
+    if let Some(market_authority) = market_authority {
+        accounts.push(AccountMeta::new_readonly(*market_authority, true));
+    }
+    Ok(Instruction {
+        program_id: *program_id,
+        data,
+        accounts,
+    })
+}
+
+pub fn prune(
+    program_id: &Pubkey,
+    market: &Pubkey,
+    bids: &Pubkey,
+    asks: &Pubkey,
+    prune_authority: &Pubkey,
+    open_orders: &Pubkey,
+    open_orders_owner: &Pubkey,
+    event_q: &Pubkey,
+    limit: u16,
+) -> Result<Instruction, DexError> {
+    let data = MarketInstruction::Prune(limit).pack();
+    let accounts: Vec<AccountMeta> = vec![
+        AccountMeta::new(*market, false),
+        AccountMeta::new(*bids, false),
+        AccountMeta::new(*asks, false),
+        AccountMeta::new_readonly(*prune_authority, true),
+        AccountMeta::new_readonly(*open_orders, false),
+        AccountMeta::new_readonly(*open_orders_owner, false),
+        AccountMeta::new(*event_q, false),
+    ];
+    Ok(Instruction {
+        program_id: *program_id,
+        data,
+        accounts,
+    })
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
